@@ -405,11 +405,8 @@ class Job:
 
         A Job can't be setup without deploying it first.
         >>> job.setup()
-        Traceback (most recent call last):
-        AttributeError: 'Job' object has no attribute 'job_contract'
+        False
 
-        >>> job.launch(rep_oracle_pub_key)
-        True
         >>> multi_credentials = [("0x61F9F0B31eacB420553da8BCC59DC617279731Ac", "486a0621e595dd7fcbe5608cbbeec8f5a8b5cabe7637f11eccfc7acd408c3a0e"), ("0x6b7E3C31F34cF38d1DFC1D9A8A59482028395809", "f22d4fc42da79aa5ba839998a0a9f2c2c45f5e55ee7f1504e464d2c71ca199e1")]
         >>> job = Job(credentials, manifest, multi_credentials=multi_credentials)
         >>> job.launch(rep_oracle_pub_key)
@@ -426,6 +423,10 @@ class Job:
             AttributeError: if trying to setup the job before deploying it.
 
         """
+
+        if not hasattr(self, 'job_contract'):
+            return False
+
         # Prepare setup arguments for the escrow contract.
         reputation_oracle_stake = int(
             Decimal(self.serialized_manifest["oracle_stake"]) * 100)
@@ -436,87 +437,61 @@ class Job:
         recording_oracle = str(
             self.serialized_manifest["recording_oracle_addr"])
         hmt_amount = int(self.amount * 10**18)
+        hmtoken_contract = get_hmtoken()
 
         hmt_transferred = False
         contract_is_setup = False
 
+        txn_event = "Transferring HMT"
         txn_info = {
             "gas_payer": self.gas_payer,
             "gas_payer_priv": self.gas_payer_priv,
             "gas": gas
         }
+        txn_func = hmtoken_contract.functions.transfer
+        func_args = [self.job_contract.address, hmt_amount]
 
-        # Fund the escrow contract with HMT.
-        hmtoken_contract = get_hmtoken()
         try:
-            txn_func = hmtoken_contract.functions.transfer
-            func_args = [self.job_contract.address, hmt_amount]
             handle_transaction(txn_func, *func_args, **txn_info)
             hmt_transferred = True
         except Exception as e:
-            LOG.error(
-                f"Transferring HMT failed with main credentials: {self.gas_payer}, {self.gas_payer_priv} due to {e}."
+            LOG.info(
+                f"{txn_event} failed with main credentials: {self.gas_payer}, {self.gas_payer_priv} due to {e}. Using secondary ones..."
             )
 
         if not hmt_transferred:
-            for gas_payer, gas_payer_priv in self.multi_credentials:
-                txn_info = {
-                    "gas_payer": gas_payer,
-                    "gas_payer_priv": gas_payer_priv,
-                    "gas": gas
-                }
+            hmt_transferred = self._raffle_txn(self.multi_credentials,
+                                               txn_func, func_args, txn_event)
 
-                try:
-                    txn_func = hmtoken_contract.functions.transfer
-                    func_args = [self.job_contract.address, hmt_amount]
-                    handle_transaction(txn_func, *func_args, **txn_info)
-                    break
-                except Exception as e:
-                    LOG.error(
-                        f"Transferring HMT failed with: {self.gas_payer} and {self.gas_payer_priv} due to {e}. Raffling new ones..."
-                    )
+        if not hmt_transferred:
+            LOG.exception(
+                f"{txn_event} failed with all credentials, not continuing to setup."
+            )
+            return False
 
-        txn_info = {
-            "gas_payer": self.gas_payer,
-            "gas_payer_priv": self.gas_payer_priv,
-            "gas": gas
-        }
+        txn_event = "Setup"
+        txn_func = self.job_contract.functions.setup
+        func_args = [
+            reputation_oracle, recording_oracle, reputation_oracle_stake,
+            recording_oracle_stake, self.manifest_url, self.manifest_hash
+        ]
 
         try:
-            # Setup the escrow contract with manifest and IPFS data.
-            txn_func = self.job_contract.functions.setup
-            func_args = [
-                reputation_oracle, recording_oracle, reputation_oracle_stake,
-                recording_oracle_stake, self.manifest_url, self.manifest_hash
-            ]
             handle_transaction(txn_func, *func_args, **txn_info)
             contract_is_setup = True
         except Exception as e:
-            LOG.error(
-                f"Setup failed with main credentials: {self.gas_payer}, {self.gas_payer_priv} due to {e}."
+            LOG.info(
+                f"{txn_event} failed with main credentials: {self.gas_payer}, {self.gas_payer_priv} due to {e}. Using secondary ones..."
             )
 
         if not contract_is_setup:
-            for gas_payer, gas_payer_priv in self.multi_credentials:
-                txn_info = {
-                    "gas_payer": gas_payer,
-                    "gas_payer_priv": gas_payer_priv,
-                    "gas": gas
-                }
-                try:
-                    txn_func = self.job_contract.functions.setup
-                    func_args = [
-                        reputation_oracle, recording_oracle,
-                        reputation_oracle_stake, recording_oracle_stake,
-                        self.manifest_url, self.manifest_hash
-                    ]
-                    handle_transaction(txn_func, *func_args, **txn_info)
-                    return self.status() == Status.Pending and self.balance(
-                    ) == hmt_amount
-                except Exception as e:
-                    LOG.error(
-                        f"Setup failed with: {gas_payer} and {gas_payer_priv} due to {e}. Raffling new ones..."
-                    )
+            contract_is_setup = self._raffle_txn(self.multi_credentials,
+                                                 txn_func, func_args,
+                                                 txn_event)
+
+        if not contract_is_setup:
+            LOG.exception(f"{txn_event} failed with all credentials.")
+
         return self.status() == Status.Pending and self.balance() == hmt_amount
 
     def add_trusted_handlers(self,
@@ -552,6 +527,7 @@ class Job:
             bool: returns True if trusted handlers have been setup successfully.
 
         """
+        txn_event = "Adding trusted handlers"
         txn_func = self.job_contract.functions.addTrustedHandlers
         txn_info = {
             "gas_payer": self.gas_payer,
@@ -559,8 +535,22 @@ class Job:
             "gas": gas
         }
         func_args = [handlers]
-        handle_transaction(txn_func, *func_args, **txn_info)
-        return True
+
+        try:
+            handle_transaction(txn_func, *func_args, **txn_info)
+            return True
+        except Exception as e:
+            LOG.info(
+                f"{txn_event} failed with main credentials: {self.gas_payer}, {self.gas_payer_priv} due to {e}. Using secondary ones..."
+            )
+        trusted_handlers_added = self._raffle_txn(self.multi_credentials,
+                                                  txn_func, func_args,
+                                                  txn_event)
+
+        if not trusted_handlers_added:
+            LOG.exception(f"{txn_event} failed with all credentials.")
+
+        return trusted_handlers_added
 
     def bulk_payout(self,
                     payouts: List[Tuple[str, Decimal]],
@@ -624,45 +614,33 @@ class Job:
             bool: returns True if paying to ethereum addresses and oracles succeeds.
 
         """
-        eth_addrs = [eth_addr for eth_addr, amount in payouts]
-        hmt_amounts = [int(amount * 10**18) for eth_addr, amount in payouts]
-
+        txn_event = "Bulk payout"
         txn_func = self.job_contract.functions.bulkPayOut
         txn_info = {
             "gas_payer": self.gas_payer,
             "gas_payer_priv": self.gas_payer_priv,
             "gas": gas
         }
+
+        (hash_, url) = upload(results, pub_key, self.use_s3_storage)
+        eth_addrs = [eth_addr for eth_addr, amount in payouts]
+        hmt_amounts = [int(amount * 10**18) for eth_addr, amount in payouts]
+
+        func_args = [eth_addrs, hmt_amounts, url, hash_, 1]
         try:
-            (hash_, url) = upload(results, pub_key, self.use_s3_storage)
-            func_args = [eth_addrs, hmt_amounts, url, hash_, 1]
             handle_transaction(txn_func, *func_args, **txn_info)
             return self._bulk_paid() == True
         except Exception as e:
-            LOG.error(
-                f"Bulk payout failed with main credentials: {self.gas_payer}, {self.gas_payer_priv} due to {e}."
+            LOG.info(
+                f"{txn_event} failed with main credentials: {self.gas_payer}, {self.gas_payer_priv} due to {e}. Using secondary ones..."
             )
 
-        bulk_paid = False
+        bulk_paid = self._raffle_txn(self.multi_credentials, txn_func,
+                                     func_args, txn_event)
+
+        if not bulk_paid:
+            LOG.exception(f"{txn_event} failed with all credentials.")
 
-        for gas_payer, gas_payer_priv in self.multi_credentials:
-            txn_info = {
-                "gas_payer": gas_payer,
-                "gas_payer_priv": gas_payer_priv,
-                "gas": gas
-            }
-            try:
-                (hash_, url) = upload(results, pub_key, self.use_s3_storage)
-                func_args = [eth_addrs, hmt_amounts, url, hash_, 1]
-                handle_transaction(txn_func, *func_args, **txn_info)
-                self.gas_payer = gas_payer
-                self.gas_payer_priv = gas_payer_priv
-                bulk_paid = True
-                break
-            except Exception as e:
-                LOG.error(
-                    f"Bulk payout failed with {gas_payer} and {gas_payer_priv} due to {e}. Raffling new ones..."
-                )
         return bulk_paid == True
 
     def abort(self, gas: int = GAS_LIMIT) -> bool:
@@ -730,6 +708,8 @@ class Job:
             bool: returns True if contract has been destroyed successfully.
 
         """
+        w3 = get_w3()
+        txn_event = "Job abortion"
         txn_func = self.job_contract.functions.abort
         txn_info = {
             "gas_payer": self.gas_payer,
@@ -737,12 +717,22 @@ class Job:
             "gas": gas
         }
 
-        handle_transaction(txn_func, *[], **txn_info)
+        try:
+            handle_transaction(txn_func, *[], **txn_info)
+            # After abort the contract should be destroyed
+            return w3.eth.getCode(self.job_contract.address) == b''
+        except Exception as e:
+            LOG.info(
+                f"{txn_event} failed with main credentials: {self.gas_payer}, {self.gas_payer_priv} due to {e}. Using secondary ones..."
+            )
 
-        # After abort the contract should be destroyed
-        w3 = get_w3()
-        contract_code = w3.eth.getCode(self.job_contract.address)
-        return contract_code == b''
+        job_aborted = self._raffle_txn(self.multi_credentials, txn_func, [],
+                                       txn_event)
+
+        if not job_aborted:
+            LOG.exception(f"{txn_event} failed with all credentials.")
+
+        return w3.eth.getCode(self.job_contract.address) == b''
 
     def cancel(self, gas: int = GAS_LIMIT) -> bool:
         """Returns the HMT back to the gas payer. It's the softer version of abort as the contract is not destroyed.
@@ -793,6 +783,7 @@ class Job:
             bool: returns True if gas payer has been paid back and contract is in "Cancelled" state.
 
         """
+        txn_event = "Job cancellation"
         txn_func = self.job_contract.functions.cancel
         txn_info = {
             "gas_payer": self.gas_payer,
@@ -800,7 +791,20 @@ class Job:
             "gas": gas
         }
 
-        handle_transaction(txn_func, *[], **txn_info)
+        try:
+            handle_transaction(txn_func, *[], **txn_info)
+            return self.status() == Status.Cancelled
+        except Exception as e:
+            LOG.info(
+                f"{txn_event} failed with main credentials: {self.gas_payer}, {self.gas_payer_priv} due to {e}. Using secondary ones..."
+            )
+
+        job_cancelled = self._raffle_txn(self.multi_credentials, txn_func, [],
+                                         txn_event)
+
+        if not job_cancelled:
+            LOG.exception(f"{txn_event} failed with all credentials.")
+
         return self.status() == Status.Cancelled
 
     def store_intermediate_results(self,
@@ -860,42 +864,30 @@ class Job:
             returns True if contract's state is updated and IPFS upload succeeds.
 
         """
+        txn_event = "Storing intermediate results"
         txn_func = self.job_contract.functions.storeResults
         txn_info = {
             "gas_payer": self.gas_payer,
             "gas_payer_priv": self.gas_payer_priv,
             "gas": gas
         }
+        (hash_, url) = upload(results, pub_key, self.use_s3_storage)
+        func_args = [url, hash_]
+
         try:
-            (hash_, url) = upload(results, pub_key, self.use_s3_storage)
-            func_args = [url, hash_]
             handle_transaction(txn_func, *func_args, **txn_info)
             return True
         except Exception as e:
-            LOG.error(
-                f"Storing intermediate results failed with main credentials: {self.gas_payer}, {self.gas_payer_priv} due to {e}."
+            LOG.info(
+                f"{txn_event} failed with main credentials: {self.gas_payer}, {self.gas_payer_priv} due to {e}. Using secondary ones..."
             )
 
-        results_stored = False
+        results_stored = self._raffle_txn(self.multi_credentials, txn_func,
+                                          func_args, txn_event)
+
+        if not results_stored:
+            LOG.exception(f"{txn_event} failed with all credentials.")
 
-        for gas_payer, gas_payer_priv in self.multi_credentials:
-            txn_info = {
-                "gas_payer": gas_payer,
-                "gas_payer_priv": gas_payer_priv,
-                "gas": gas
-            }
-            try:
-                (hash_, url) = upload(results, pub_key, self.use_s3_storage)
-                func_args = [url, hash_]
-                handle_transaction(txn_func, *func_args, **txn_info)
-                self.gas_payer = gas_payer
-                self.gas_payer_priv = gas_payer_priv
-                results_stored = True
-                break
-            except Exception as e:
-                LOG.error(
-                    f"Storing intermediate results failed with {gas_payer} and {gas_payer_priv} due to {e}. Raffling new ones..."
-                )
         return results_stored
 
     def complete(self, gas: int = GAS_LIMIT) -> bool:
@@ -934,6 +926,7 @@ class Job:
             bool: returns True if the contract has been completed.
 
         """
+        txn_event = "Job completion"
         txn_func = self.job_contract.functions.complete
         txn_info = {
             "gas_payer": self.gas_payer,
@@ -941,7 +934,20 @@ class Job:
             "gas": gas
         }
 
-        handle_transaction(txn_func, *[], **txn_info)
+        try:
+            handle_transaction(txn_func, *[], **txn_info)
+            return self.status() == Status.Complete
+        except Exception as e:
+            LOG.info(
+                f"{txn_event} failed with main credentials: {self.gas_payer}, {self.gas_payer_priv} due to {e}. Using secondary ones..."
+            )
+
+        job_completed = self._raffle_txn(self.multi_credentials, txn_func, [],
+                                         txn_event)
+
+        if not job_completed:
+            LOG.exception(f"{txn_event} failed with all credentials.")
+
         return self.status() == Status.Complete
 
     def status(self, gas: int = GAS_LIMIT) -> Enum:
@@ -1400,43 +1406,70 @@ class Job:
             TimeoutError: if wait_on_transaction times out.
 
         """
-        try:
-            txn_func = self.factory_contract.functions.createEscrow
-            txn_args = [trusted_handlers]
-            txn_info = {
-                "gas_payer": self.gas_payer,
-                "gas_payer_priv": self.gas_payer_priv,
-                "gas": gas
-            }
+        txn_event = "Contract creation"
+        txn_func = self.factory_contract.functions.createEscrow
+        txn_info = {
+            "gas_payer": self.gas_payer,
+            "gas_payer_priv": self.gas_payer_priv,
+            "gas": gas
+        }
+        func_args = [trusted_handlers]
 
-            handle_transaction(txn_func, *txn_args, **txn_info)
+        try:
+            handle_transaction(txn_func, *func_args, **txn_info)
             return True
         except Exception as e:
-            LOG.error(
-                f"Contract creation failed with main credentials: {self.gas_payer}, {self.gas_payer_priv} due to {e}."
+            LOG.info(
+                f"{txn_event} failed with main credentials: {self.gas_payer}, {self.gas_payer_priv} due to {e}. Using secondary ones..."
             )
 
-        escrow_created = False
+        escrow_created = self._raffle_txn(self.multi_credentials, txn_func,
+                                          func_args, txn_event)
 
-        for gas_payer, gas_payer_priv in self.multi_credentials:
+        if not escrow_created:
+            LOG.exception(f"{txn_event} failed with all credentials.")
+
+        return escrow_created
+
+    def _raffle_txn(self,
+                    multi_creds,
+                    txn_func,
+                    txn_args,
+                    txn_event,
+                    gas: int = GAS_LIMIT):
+        """Takes in multiple credentials, loops through each and performs the given transaction.
+
+        Args:
+            credentials (Dict[str, str]): a dict of multiple ethereum addresses and their private keys.
+            txn_func: the transaction function to be handled.
+            txn_args (List): the arguments the transaction takes.
+            txn_event (str): the transaction event that will be performed.
+            gas (int): maximum amount of gas the caller is ready to pay.
+        
+        Returns:
+            bool: returns True if the given transaction succeeds.
+
+        """
+        txn_succeeded = False
+
+        for gas_payer, gas_payer_priv in multi_creds:
             txn_info = {
                 "gas_payer": gas_payer,
                 "gas_payer_priv": gas_payer_priv,
                 "gas": gas
             }
-            txn_args = [trusted_handlers]
             try:
                 handle_transaction(txn_func, *txn_args, **txn_info)
                 self.gas_payer = gas_payer
                 self.gas_payer_priv = gas_payer_priv
-                escrow_created = True
+                txn_succeeded = True
                 break
             except Exception as e:
-                LOG.error(
-                    f"Contract creation failed with {gas_payer} and {gas_payer_priv} due to {e}. Raffling new ones..."
+                LOG.info(
+                    f"{txn_event} failed with {gas_payer} and {gas_payer_priv} due to {e}."
                 )
 
-        return escrow_created
+        return txn_succeeded
 
 
 if __name__ == "__main__":
